@@ -1,5 +1,5 @@
 /* root-dev-ino.c -- get the device and inode numbers for '/'.
-   Copyright (C) 2003-2022 Free Software Foundation, Inc.
+   Copyright (C) 2003-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,18 +23,16 @@
 #include <stdlib.h>
 
 /* Call lstat to get the device and inode numbers for '/'.
-   Upon failure, return NULL.  Otherwise, set the members of
+   Upon failure, return nullptr.  Otherwise, set the members of
    *ROOT_D_I accordingly and return ROOT_D_I.  */
 struct root_dev_ino *
 get_root_dev_ino (struct root_dev_ino *root_d_i)
 {
   struct stat statbuf;
-  if (lstat ("/", &statbuf))
-    return NULL;
+  if (lstat ("/", &statbuf) || lstat ("//", &statbuf))
+    return nullptr;
   root_d_i->single_slash.st_ino = statbuf.st_ino;
   root_d_i->single_slash.st_dev = statbuf.st_dev;
-  if (lstat ("//", &statbuf))
-    return NULL;
   root_d_i->double_slash.st_ino = statbuf.st_ino;
   root_d_i->double_slash.st_dev = statbuf.st_dev;
   return root_d_i;
