@@ -1,5 +1,5 @@
 /* dircolors - output commands to set the LS_COLOR environment variable
-   Copyright (C) 1996-2022 Free Software Foundation, Inc.
+   Copyright (C) 1996-2024 Free Software Foundation, Inc.
    Copyright (C) 1994, 1995, 1997, 1998, 1999, 2000 H. Peter Anvin
 
    This program is free software: you can redistribute it and/or modify
@@ -23,9 +23,8 @@
 
 #include "system.h"
 #include "dircolors.h"
+#include "c-ctype.h"
 #include "c-strcase.h"
-#include "die.h"
-#include "error.h"
 #include "obstack.h"
 #include "quote.h"
 #include "stdio--.h"
@@ -58,16 +57,17 @@ static char const *const slack_codes[] =
   "CHR", "CHAR", "DOOR", "EXEC", "LEFT", "LEFTCODE", "RIGHT", "RIGHTCODE",
   "END", "ENDCODE", "SUID", "SETUID", "SGID", "SETGID", "STICKY",
   "OTHER_WRITABLE", "OWR", "STICKY_OTHER_WRITABLE", "OWT", "CAPABILITY",
-  "MULTIHARDLINK", "CLRTOEOL", NULL
+  "MULTIHARDLINK", "CLRTOEOL", nullptr
 };
 
 static char const *const ls_codes[] =
 {
   "no", "no", "fi", "rs", "di", "ln", "ln", "ln", "or", "mi", "pi", "pi",
   "so", "bd", "bd", "cd", "cd", "do", "ex", "lc", "lc", "rc", "rc", "ec", "ec",
-  "su", "su", "sg", "sg", "st", "ow", "ow", "tw", "tw", "ca", "mh", "cl", NULL
+  "su", "su", "sg", "sg", "st", "ow", "ow", "tw", "tw", "ca", "mh", "cl",
+  nullptr
 };
-verify (ARRAY_CARDINALITY (slack_codes) == ARRAY_CARDINALITY (ls_codes));
+static_assert (ARRAY_CARDINALITY (slack_codes) == ARRAY_CARDINALITY (ls_codes));
 
 /* Whether to output escaped ls color codes for display.  */
 static bool print_ls_colors;
@@ -81,15 +81,15 @@ enum
 
 static struct option const long_options[] =
   {
-    {"bourne-shell", no_argument, NULL, 'b'},
-    {"sh", no_argument, NULL, 'b'},
-    {"csh", no_argument, NULL, 'c'},
-    {"c-shell", no_argument, NULL, 'c'},
-    {"print-database", no_argument, NULL, 'p'},
-    {"print-ls-colors", no_argument, NULL, PRINT_LS_COLORS_OPTION},
+    {"bourne-shell", no_argument, nullptr, 'b'},
+    {"sh", no_argument, nullptr, 'b'},
+    {"csh", no_argument, nullptr, 'c'},
+    {"c-shell", no_argument, nullptr, 'c'},
+    {"print-database", no_argument, nullptr, 'p'},
+    {"print-ls-colors", no_argument, nullptr, PRINT_LS_COLORS_OPTION},
     {GETOPT_HELP_OPTION_DECL},
     {GETOPT_VERSION_OPTION_DECL},
-    {NULL, 0, NULL, 0}
+    {nullptr, 0, nullptr, 0}
   };
 
 void
@@ -132,7 +132,7 @@ guess_shell_syntax (void)
   char *shell;
 
   shell = getenv ("SHELL");
-  if (shell == NULL || *shell == '\0')
+  if (shell == nullptr || *shell == '\0')
     return SHELL_SYNTAX_UNKNOWN;
 
   shell = last_component (shell);
@@ -150,10 +150,10 @@ parse_line (char const *line, char **keyword, char **arg)
   char const *keyword_start;
   char const *arg_start;
 
-  *keyword = NULL;
-  *arg = NULL;
+  *keyword = nullptr;
+  *arg = nullptr;
 
-  for (p = line; isspace (to_uchar (*p)); ++p)
+  for (p = line; c_isspace (to_uchar (*p)); ++p)
     continue;
 
   /* Ignore blank lines and shell-style comments.  */
@@ -162,7 +162,7 @@ parse_line (char const *line, char **keyword, char **arg)
 
   keyword_start = p;
 
-  while (!isspace (to_uchar (*p)) && *p != '\0')
+  while (!c_isspace (to_uchar (*p)) && *p != '\0')
     {
       ++p;
     }
@@ -175,7 +175,7 @@ parse_line (char const *line, char **keyword, char **arg)
     {
       ++p;
     }
-  while (isspace (to_uchar (*p)));
+  while (c_isspace (to_uchar (*p)));
 
   if (*p == '\0' || *p == '#')
     return;
@@ -185,7 +185,7 @@ parse_line (char const *line, char **keyword, char **arg)
   while (*p != '\0' && *p != '#')
     ++p;
 
-  for (--p; isspace (to_uchar (*p)); --p)
+  for (--p; c_isspace (to_uchar (*p)); --p)
     continue;
   ++p;
 
@@ -265,9 +265,9 @@ append_entry (char prefix, char const *item, char const *arg)
 static bool
 dc_parse_stream (FILE *fp, char const *filename)
 {
-  size_t line_number = 0;
+  idx_t line_number = 0;
   char const *next_G_line = G_line;
-  char *input_line = NULL;
+  char *input_line = nullptr;
   size_t input_line_size = 0;
   char const *line;
   char const *term;
@@ -279,12 +279,12 @@ dc_parse_stream (FILE *fp, char const *filename)
 
   /* Get terminal type */
   term = getenv ("TERM");
-  if (term == NULL || *term == '\0')
+  if (term == nullptr || *term == '\0')
     term = "none";
 
   /* Also match $COLORTERM.  */
   colorterm = getenv ("COLORTERM");
-  if (colorterm == NULL)
+  if (colorterm == nullptr)
     colorterm = "";  /* Doesn't match default "?*"  */
 
   while (true)
@@ -298,6 +298,11 @@ dc_parse_stream (FILE *fp, char const *filename)
         {
           if (getline (&input_line, &input_line_size, fp) <= 0)
             {
+              if (ferror (fp))
+                {
+                  error (0, errno, _("%s: read error"), quotef (filename));
+                  ok = false;
+                }
               free (input_line);
               break;
             }
@@ -313,13 +318,13 @@ dc_parse_stream (FILE *fp, char const *filename)
 
       parse_line (line, &keywd, &arg);
 
-      if (keywd == NULL)
+      if (keywd == nullptr)
         continue;
 
-      if (arg == NULL)
+      if (arg == nullptr)
         {
-          error (0, 0, _("%s:%lu: invalid line;  missing second token"),
-                 quotef (filename), (unsigned long int) line_number);
+          error (0, 0, _("%s:%td: invalid line;  missing second token"),
+                 quotef (filename), line_number);
           ok = false;
           free (keywd);
           continue;
@@ -357,11 +362,11 @@ dc_parse_stream (FILE *fp, char const *filename)
                 {
                   int i;
 
-                  for (i = 0; slack_codes[i] != NULL; ++i)
+                  for (i = 0; slack_codes[i] != nullptr; ++i)
                     if (c_strcasecmp (keywd, slack_codes[i]) == 0)
                       break;
 
-                  if (slack_codes[i] != NULL)
+                  if (slack_codes[i] != nullptr)
                     append_entry (0, ls_codes[i], arg);
                   else
                     unrecognized = true;
@@ -373,9 +378,9 @@ dc_parse_stream (FILE *fp, char const *filename)
 
       if (unrecognized && (state == ST_TERMSURE || state == ST_TERMYES))
         {
-          error (0, 0, _("%s:%lu: unrecognized keyword %s"),
+          error (0, 0, _("%s:%td: unrecognized keyword %s"),
                  (filename ? quotef (filename) : _("<internal>")),
-                 (unsigned long int) line_number, keywd);
+                 line_number, keywd);
           ok = false;
         }
 
@@ -391,7 +396,7 @@ dc_parse_file (char const *filename)
 {
   bool ok;
 
-  if (! STREQ (filename, "-") && freopen (filename, "r", stdin) == NULL)
+  if (! STREQ (filename, "-") && freopen (filename, "r", stdin) == nullptr)
     {
       error (0, errno, "%s", quotef (filename));
       return false;
@@ -424,7 +429,7 @@ main (int argc, char **argv)
 
   atexit (close_stdout);
 
-  while ((optc = getopt_long (argc, argv, "bcp", long_options, NULL)) != -1)
+  while ((optc = getopt_long (argc, argv, "bcp", long_options, nullptr)) != -1)
     switch (optc)
       {
       case 'b':	/* Bourne shell syntax.  */
@@ -499,15 +504,14 @@ main (int argc, char **argv)
         {
           syntax = guess_shell_syntax ();
           if (syntax == SHELL_SYNTAX_UNKNOWN)
-            {
-              die (EXIT_FAILURE, 0,
-         _("no SHELL environment variable, and no shell type option given"));
-            }
+            error (EXIT_FAILURE, 0,
+                   _("no SHELL environment variable,"
+                     " and no shell type option given"));
         }
 
       obstack_init (&lsc_obstack);
       if (argc == 0)
-        ok = dc_parse_stream (NULL, NULL);
+        ok = dc_parse_stream (nullptr, nullptr);
       else
         ok = dc_parse_file (argv[0]);
 
