@@ -1,5 +1,5 @@
 /* 'rm' file deletion utility for GNU.
-   Copyright (C) 1988-2022 Free Software Foundation, Inc.
+   Copyright (C) 1988-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,12 +22,10 @@
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
-#include <assert.h>
 
 #include "system.h"
 #include "argmatch.h"
-#include "die.h"
-#include "error.h"
+#include "assure.h"
 #include "remove.h"
 #include "root-dev-ino.h"
 #include "yesno.h"
@@ -62,32 +60,32 @@ enum interactive_type
 
 static struct option const long_opts[] =
 {
-  {"force", no_argument, NULL, 'f'},
-  {"interactive", optional_argument, NULL, INTERACTIVE_OPTION},
+  {"force", no_argument, nullptr, 'f'},
+  {"interactive", optional_argument, nullptr, INTERACTIVE_OPTION},
 
-  {"one-file-system", no_argument, NULL, ONE_FILE_SYSTEM},
-  {"no-preserve-root", no_argument, NULL, NO_PRESERVE_ROOT},
-  {"preserve-root", optional_argument, NULL, PRESERVE_ROOT},
+  {"one-file-system", no_argument, nullptr, ONE_FILE_SYSTEM},
+  {"no-preserve-root", no_argument, nullptr, NO_PRESERVE_ROOT},
+  {"preserve-root", optional_argument, nullptr, PRESERVE_ROOT},
 
   /* This is solely for testing.  Do not document.  */
   /* It is relatively difficult to ensure that there is a tty on stdin.
      Since rm acts differently depending on that, without this option,
      it'd be harder to test the parts of rm that depend on that setting.  */
-  {"-presume-input-tty", no_argument, NULL, PRESUME_INPUT_TTY_OPTION},
+  {"-presume-input-tty", no_argument, nullptr, PRESUME_INPUT_TTY_OPTION},
 
-  {"recursive", no_argument, NULL, 'r'},
-  {"dir", no_argument, NULL, 'd'},
-  {"verbose", no_argument, NULL, 'v'},
+  {"recursive", no_argument, nullptr, 'r'},
+  {"dir", no_argument, nullptr, 'd'},
+  {"verbose", no_argument, nullptr, 'v'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {NULL, 0, NULL, 0}
+  {nullptr, 0, nullptr, 0}
 };
 
 static char const *const interactive_args[] =
 {
   "never", "no", "none",
   "once",
-  "always", "yes", NULL
+  "always", "yes", nullptr
 };
 static enum interactive_type const interactive_types[] =
 {
@@ -166,6 +164,11 @@ Remove (unlink) the FILE(s).\n\
 \n\
 By default, rm does not remove directories.  Use the --recursive (-r or -R)\n\
 option to remove each listed directory, too, along with all of its contents.\n\
+"), stdout);
+      fputs (_("\
+\n\
+Any attempt to remove a file whose last file name component is '.' or '..'\n\
+is rejected with a diagnostic.\n\
 "), stdout);
       printf (_("\
 \n\
@@ -178,9 +181,9 @@ use one of these commands:\n\
               program_name, program_name);
       fputs (_("\
 \n\
-Note that if you use rm to remove a file, it might be possible to recover\n\
+If you use rm to remove a file, it might be possible to recover\n\
 some of its contents, given sufficient expertise and/or time.  For greater\n\
-assurance that the contents are truly unrecoverable, consider using shred(1).\n\
+assurance that the contents are unrecoverable, consider using shred(1).\n\
 "), stdout);
       emit_ancillary_info (PROGRAM_NAME);
     }
@@ -195,7 +198,7 @@ rm_option_init (struct rm_options *x)
   x->one_file_system = false;
   x->remove_empty_directories = false;
   x->recursive = false;
-  x->root_dev_ino = NULL;
+  x->root_dev_ino = nullptr;
   x->preserve_all_root = false;
   x->stdin_tty = isatty (STDIN_FILENO);
   x->verbose = false;
@@ -226,7 +229,7 @@ main (int argc, char **argv)
   /* Try to disable the ability to unlink a directory.  */
   priv_set_remove_linkdir ();
 
-  while ((c = getopt_long (argc, argv, "dfirvIR", long_opts, NULL)) != -1)
+  while ((c = getopt_long (argc, argv, "dfirvIR", long_opts, nullptr)) != -1)
     {
       switch (c)
         {
@@ -293,8 +296,8 @@ main (int argc, char **argv)
 
         case NO_PRESERVE_ROOT:
           if (! STREQ (argv[optind - 1], "--no-preserve-root"))
-            die (EXIT_FAILURE, 0,
-                 _("you may not abbreviate the --no-preserve-root option"));
+            error (EXIT_FAILURE, 0,
+                   _("you may not abbreviate the --no-preserve-root option"));
           preserve_root = false;
           break;
 
@@ -304,11 +307,9 @@ main (int argc, char **argv)
               if STREQ (optarg, "all")
                 x.preserve_all_root = true;
               else
-                {
-                  die (EXIT_FAILURE, 0,
+                error (EXIT_FAILURE, 0,
                        _("unrecognized --preserve-root argument: %s"),
                        quoteaf (optarg));
-                }
             }
           preserve_root = true;
           break;
@@ -344,9 +345,9 @@ main (int argc, char **argv)
     {
       static struct root_dev_ino dev_ino_buf;
       x.root_dev_ino = get_root_dev_ino (&dev_ino_buf);
-      if (x.root_dev_ino == NULL)
-        die (EXIT_FAILURE, errno, _("failed to get attributes of %s"),
-             quoteaf ("/"));
+      if (x.root_dev_ino == nullptr)
+        error (EXIT_FAILURE, errno, _("failed to get attributes of %s"),
+               quoteaf ("/"));
     }
 
   uintmax_t n_files = argc - optind;
@@ -356,11 +357,11 @@ main (int argc, char **argv)
     {
       fprintf (stderr,
                (x.recursive
-                ? ngettext ("%s: remove %"PRIuMAX" argument recursively? ",
-                            "%s: remove %"PRIuMAX" arguments recursively? ",
+                ? ngettext ("%s: remove %ju argument recursively? ",
+                            "%s: remove %ju arguments recursively? ",
                             select_plural (n_files))
-                : ngettext ("%s: remove %"PRIuMAX" argument? ",
-                            "%s: remove %"PRIuMAX" arguments? ",
+                : ngettext ("%s: remove %ju argument? ",
+                            "%s: remove %ju arguments? ",
                             select_plural (n_files))),
                program_name, n_files);
       if (!yesno ())
@@ -368,6 +369,6 @@ main (int argc, char **argv)
     }
 
   enum RM_status status = rm (file, &x);
-  assert (VALID_STATUS (status));
+  affirm (VALID_STATUS (status));
   return status == RM_ERROR ? EXIT_FAILURE : EXIT_SUCCESS;
 }
