@@ -1,5 +1,5 @@
 /* stat.c -- display file or file system status
-   Copyright (C) 2001-2022 Free Software Foundation, Inc.
+   Copyright (C) 2001-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,9 +28,7 @@
 # define USE_STATVFS 0
 #endif
 
-#include <stddef.h>
 #include <stdio.h>
-#include <stdalign.h>
 #include <sys/types.h>
 #include <pwd.h>
 #include <grp.h>
@@ -55,17 +53,16 @@
 # include <fs_info.h>
 #endif
 #include <selinux/selinux.h>
+#include <getopt.h>
 
 #include "system.h"
 
 #include "areadlink.h"
 #include "argmatch.h"
-#include "die.h"
-#include "error.h"
+#include "c-ctype.h"
 #include "file-type.h"
 #include "filemode.h"
 #include "fs.h"
-#include "getopt.h"
 #include "mountlist.h"
 #include "quote.h"
 #include "stat-size.h"
@@ -186,7 +183,7 @@ statfs (char const *filename, struct fs_info *buf)
 static char const digits[] = "0123456789";
 
 /* Flags that are portable for use in printf, for at least one
-   conversion specifier; make_format removes unportable flags as
+   conversion specifier; make_format removes non-portable flags as
    needed for particular specifiers.  The glibc 2.2 extension "I" is
    listed here; it is removed by make_format because it has undefined
    behavior elsewhere and because it is incompatible with
@@ -218,7 +215,7 @@ enum cached_mode
 
 static char const *const cached_args[] =
 {
-  "default", "never", "always", NULL
+  "default", "never", "always", nullptr
 };
 
 static enum cached_mode const cached_modes[] =
@@ -228,18 +225,18 @@ static enum cached_mode const cached_modes[] =
 
 static struct option const long_options[] =
 {
-  {"dereference", no_argument, NULL, 'L'},
-  {"file-system", no_argument, NULL, 'f'},
-  {"format", required_argument, NULL, 'c'},
-  {"printf", required_argument, NULL, PRINTF_OPTION},
-  {"terse", no_argument, NULL, 't'},
-  {"cached", required_argument, NULL, 0},
+  {"dereference", no_argument, nullptr, 'L'},
+  {"file-system", no_argument, nullptr, 'f'},
+  {"format", required_argument, nullptr, 'c'},
+  {"printf", required_argument, nullptr, PRINTF_OPTION},
+  {"terse", no_argument, nullptr, 't'},
+  {"cached", required_argument, nullptr, 0},
 #ifdef __CYGWIN__
-  {"append-exe", no_argument, NULL, APPEND_EXE_OPTION},
+  {"append-exe", no_argument, nullptr, APPEND_EXE_OPTION},
 #endif
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {NULL, 0, NULL, 0}
+  {nullptr, 0, nullptr, 0}
 };
 
 /* Whether to follow symbolic links;  True for --dereference (-L).  */
@@ -707,25 +704,25 @@ out_string (char *pformat, size_t prefix_len, char const *arg)
 static int
 out_int (char *pformat, size_t prefix_len, intmax_t arg)
 {
-  make_format (pformat, prefix_len, "'-+ 0", PRIdMAX);
+  make_format (pformat, prefix_len, "'-+ 0", "jd");
   return printf (pformat, arg);
 }
 static int
 out_uint (char *pformat, size_t prefix_len, uintmax_t arg)
 {
-  make_format (pformat, prefix_len, "'-0", PRIuMAX);
+  make_format (pformat, prefix_len, "'-0", "ju");
   return printf (pformat, arg);
 }
 static void
 out_uint_o (char *pformat, size_t prefix_len, uintmax_t arg)
 {
-  make_format (pformat, prefix_len, "-#0", PRIoMAX);
+  make_format (pformat, prefix_len, "-#0", "jo");
   printf (pformat, arg);
 }
 static void
 out_uint_x (char *pformat, size_t prefix_len, uintmax_t arg)
 {
-  make_format (pformat, prefix_len, "-#0", PRIxMAX);
+  make_format (pformat, prefix_len, "-#0", "jx");
   printf (pformat, arg);
 }
 static int
@@ -754,7 +751,7 @@ out_epoch_sec (char *pformat, size_t prefix_len,
 
       if (ISDIGIT (dot[1]))
         {
-          long int lprec = strtol (dot + 1, NULL, 10);
+          long int lprec = strtol (dot + 1, nullptr, 10);
           precision = (lprec <= INT_MAX ? lprec : INT_MAX);
         }
       else
@@ -774,7 +771,7 @@ out_epoch_sec (char *pformat, size_t prefix_len,
             --p;
           while (ISDIGIT (p[-1]));
 
-          long int lwidth = strtol (p, NULL, 10);
+          long int lwidth = strtol (p, nullptr, 10);
           width = (lwidth <= INT_MAX ? lwidth : INT_MAX);
           if (1 < width)
             {
@@ -857,7 +854,7 @@ out_file_context (char *pformat, size_t prefix_len, char const *filename)
     {
       error (0, errno, _("failed to get security context of %s"),
              quoteaf (filename));
-      scontext = NULL;
+      scontext = nullptr;
       fail = true;
     }
   strcpy (pformat + prefix_len, "s");
@@ -889,9 +886,10 @@ print_statfs (char *pformat, size_t prefix_len, MAYBE_UNUSED char mod, char m,
         uintmax_t fsid = statfsbuf->f_fsid;
 #else
         typedef unsigned int fsid_word;
-        verify (alignof (STRUCT_STATVFS) % alignof (fsid_word) == 0);
-        verify (offsetof (STRUCT_STATVFS, f_fsid) % alignof (fsid_word) == 0);
-        verify (sizeof statfsbuf->f_fsid % alignof (fsid_word) == 0);
+        static_assert (alignof (STRUCT_STATVFS) % alignof (fsid_word) == 0);
+        static_assert (offsetof (STRUCT_STATVFS, f_fsid) % alignof (fsid_word)
+                       == 0);
+        static_assert (sizeof statfsbuf->f_fsid % alignof (fsid_word) == 0);
         fsid_word const *p = (fsid_word *) &statfsbuf->f_fsid;
 
         /* Assume a little-endian word order, as that is compatible
@@ -956,12 +954,12 @@ print_statfs (char *pformat, size_t prefix_len, MAYBE_UNUSED char mod, char m,
 
 /* Return any bind mounted source for a path.
    The caller should not free the returned buffer.
-   Return NULL if no bind mount found.  */
+   Return nullptr if no bind mount found.  */
 NODISCARD
 static char const *
 find_bind_mount (char const * name)
 {
-  char const * bind_mount = NULL;
+  char const * bind_mount = nullptr;
 
   static struct mount_entry *mount_list;
   static bool tried_mount_list = false;
@@ -974,7 +972,7 @@ find_bind_mount (char const * name)
 
   struct stat name_stats;
   if (stat (name, &name_stats) != 0)
-    return NULL;
+    return nullptr;
 
   struct mount_entry *me;
   for (me = mount_list; me; me = me->me_next)
@@ -985,7 +983,7 @@ find_bind_mount (char const * name)
           struct stat dev_stats;
 
           if (stat (me->me_devname, &dev_stats) == 0
-              && SAME_INODE (name_stats, dev_stats))
+              && psame_inode (&name_stats, &dev_stats))
             {
               bind_mount = me->me_devname;
               break;
@@ -1003,8 +1001,8 @@ out_mount_point (char const *filename, char *pformat, size_t prefix_len,
                  const struct stat *statp)
 {
 
-  char const *np = "?", *bp = NULL;
-  char *mp = NULL;
+  char const *np = "?", *bp = nullptr;
+  char *mp = nullptr;
   bool fail = true;
 
   /* Look for bind mounts first.  Note we output the immediate alias,
@@ -1052,7 +1050,7 @@ neg_to_zero (struct timespec ts)
 {
   if (0 <= ts.tv_nsec)
     return ts;
-  struct timespec z = {0, 0};
+  struct timespec z = {0};
   return z;
 }
 
@@ -1067,20 +1065,20 @@ getenv_quoting_style (void)
     {
       int i = ARGMATCH (q_style, quoting_style_args, quoting_style_vals);
       if (0 <= i)
-        set_quoting_style (NULL, quoting_style_vals[i]);
+        set_quoting_style (nullptr, quoting_style_vals[i]);
       else
         {
-          set_quoting_style (NULL, shell_escape_always_quoting_style);
+          set_quoting_style (nullptr, shell_escape_always_quoting_style);
           error (0, 0, _("ignoring invalid value of environment "
                          "variable QUOTING_STYLE: %s"), quote (q_style));
         }
     }
   else
-    set_quoting_style (NULL, shell_escape_always_quoting_style);
+    set_quoting_style (nullptr, shell_escape_always_quoting_style);
 }
 
 /* Equivalent to quotearg(), but explicit to avoid syntax checks.  */
-#define quoteN(x) quotearg_style (get_quoting_style (NULL), x)
+#define quoteN(x) quotearg_style (get_quoting_style (nullptr), x)
 
 /* Output a single-character \ escape.  */
 
@@ -1152,8 +1150,8 @@ print_it (char const *format, int fd, char const *filename,
   enum
     {
       MAX_ADDITIONAL_BYTES =
-        (MAX (sizeof PRIdMAX,
-              MAX (sizeof PRIoMAX, MAX (sizeof PRIuMAX, sizeof PRIxMAX)))
+        (MAX (sizeof "jd",
+              MAX (sizeof "jo", MAX (sizeof "ju", sizeof "jx")))
          - 1)
     };
   size_t n_alloc = strlen (format) + MAX_ADDITIONAL_BYTES + 1;
@@ -1181,8 +1179,8 @@ print_it (char const *format, int fd, char const *filename,
                   {
                     dest[len] = fmt_char;
                     dest[len + 1] = '\0';
-                    die (EXIT_FAILURE, 0, _("%s: invalid directive"),
-                         quote (dest));
+                    error (EXIT_FAILURE, 0, _("%s: invalid directive"),
+                           quote (dest));
                   }
                 putchar ('%');
                 break;
@@ -1228,13 +1226,13 @@ print_it (char const *format, int fd, char const *filename,
               putchar (esc_value);
               --b;
             }
-          else if (*b == 'x' && isxdigit (to_uchar (b[1])))
+          else if (*b == 'x' && c_isxdigit (to_uchar (b[1])))
             {
               int esc_value = hextobin (b[1]);	/* Value of \xhh escape. */
               /* A hexadecimal \xhh escape sequence must have
                  1 or 2 hex. digits.  */
               ++b;
-              if (isxdigit (to_uchar (b[1])))
+              if (c_isxdigit (to_uchar (b[1])))
                 {
                   ++b;
                   esc_value = esc_value * 16 + hextobin (*b);
@@ -1281,7 +1279,7 @@ do_statfs (char const *filename, char const *format)
     }
 
 #ifdef __CYGWIN__
-  char *name_alt = NULL;
+  char *name_alt = nullptr;
   if (append_exe && 0 < cygwin_spelling (filename))
     CYGWIN_APPEND_EXE (name_alt, filename);
 #endif /* __CYGWIN__ */
@@ -1395,11 +1393,11 @@ do_stat (char const *filename, char const *format, char const *format2)
   int fd = STREQ (filename, "-") ? 0 : AT_FDCWD;
   int flags = 0;
   struct stat st;
-  struct statx stx = { 0, };
+  struct statx stx = {0};
   char const *pathname = filename;
   struct print_args pa;
   pa.st = &st;
-  pa.btime = (struct timespec) {-1, -1};
+  pa.btime = (struct timespec) {.tv_sec = -1, .tv_nsec = -1};
 
   if (AT_FDCWD != fd)
     {
@@ -1484,9 +1482,9 @@ do_stat (char const *filename, char const *format,
   int fd = STREQ (filename, "-") ? 0 : -1;
   struct stat statbuf;
   struct print_args pa;
-  char *name_alt = NULL;
+  char *name_alt = nullptr;
   pa.st = &statbuf;
-  pa.btime = (struct timespec) {-1, -1};
+  pa.btime = (struct timespec) {.tv_sec = -1, .tv_nsec = -1};
 
   if (0 <= fd)
     {
@@ -1504,10 +1502,10 @@ do_stat (char const *filename, char const *format,
       if ((follow_links
 	   ? stat (filename, &statbuf)
 	   : lstat (filename, &statbuf)) != 0)
-	{
-	  error (0, errno, _("cannot stat %s"), quoteaf (filename));
-	  return false;
-	}
+	 {
+	   error (0, errno, _("cannot stat %s"), quoteaf (filename));
+	   return false;
+	 }
 #ifdef __CYGWIN__
       if (append_exe && 0 < cygwin_spelling (filename))
         CYGWIN_APPEND_EXE (name_alt, filename);
@@ -1525,6 +1523,14 @@ do_stat (char const *filename, char const *format,
 }
 #endif /* USE_STATX */
 
+/* POSIX requires 'ls' to print file sizes without a sign, even
+   when negative.  Be consistent with that.  */
+
+static uintmax_t
+unsigned_file_size (off_t size)
+{
+  return size + (size < 0) * ((uintmax_t) OFF_T_MAX - OFF_T_MIN + 1);
+}
 
 /* Print stat info.  Return zero upon success, nonzero upon failure.  */
 static bool
@@ -1548,7 +1554,7 @@ print_stat (char *pformat, size_t prefix_len, char mod, char m,
       if (S_ISLNK (statbuf->st_mode))
         {
           char *linkname = areadlink_with_size (filename, statbuf->st_size);
-          if (linkname == NULL)
+          if (linkname == nullptr)
             {
               error (0, errno, _("cannot read symbolic link %s"),
                      quoteaf (filename));
@@ -1608,7 +1614,7 @@ print_stat (char *pformat, size_t prefix_len, char mod, char m,
       fail |= out_mount_point (filename, pformat, prefix_len, statbuf);
       break;
     case 's':
-      out_int (pformat, prefix_len, statbuf->st_size);
+      out_uint (pformat, prefix_len, unsigned_file_size (statbuf->st_size));
       break;
     case 'r':
       if (mod == 'H')
@@ -1631,10 +1637,10 @@ print_stat (char *pformat, size_t prefix_len, char mod, char m,
       out_uint (pformat, prefix_len, ST_NBLOCKSIZE);
       break;
     case 'b':
-      out_uint (pformat, prefix_len, ST_NBLOCKS (*statbuf));
+      out_uint (pformat, prefix_len, STP_NBLOCKS (statbuf));
       break;
     case 'o':
-      out_uint (pformat, prefix_len, ST_BLKSIZE (*statbuf));
+      out_uint (pformat, prefix_len, STP_BLKSIZE (statbuf));
       break;
     case 'w':
       {
@@ -1821,7 +1827,7 @@ The MODE argument of --cached can be: always, never, or default.\n\
       fputs (_("\n\
 The valid format sequences for files (without --file-system):\n\
 \n\
-  %a   permission bits in octal (note '#' and '0' printf flags)\n\
+  %a   permission bits in octal (see '#' and '0' printf flags)\n\
   %A   permission bits and file type in human readable form\n\
   %b   number of blocks allocated (see %B)\n\
   %B   the size in bytes of each block reported by %b\n\
@@ -1913,7 +1919,7 @@ main (int argc, char *argv[])
   int c;
   bool fs = false;
   bool terse = false;
-  char *format = NULL;
+  char *format = nullptr;
   char *format2;
   bool ok = true;
 
@@ -1929,7 +1935,7 @@ main (int argc, char *argv[])
 
   atexit (close_stdout);
 
-  while ((c = getopt_long (argc, argv, "c:fLt", long_options, NULL)) != -1)
+  while ((c = getopt_long (argc, argv, "c:fLt", long_options, nullptr)) != -1)
     {
       switch (c)
         {
